@@ -57,8 +57,7 @@ dropCommonPrefix <- function (strings)
 #' 
 #' J.D. Clayden, S. Muñoz Maniega, A.J. Storkey, M.D. King, M.E. Bastin & C.A.
 #' Clark (2011). TractoR: Magnetic resonance imaging and tractography with R.
-#' Journal of Statistical Software 44(8):1-18.
-#' \url{https://www.jstatsoft.org/v44/i08/}.
+#' Journal of Statistical Software 44(8):1-18. \doi{10.18637/jss.v044.i08}.
 #' @export
 sortDicomDirectories <- function (directories, method = c("internal","divest"), deleteOriginals = FALSE, sortOn = "series", seriesId = c("UID","number","time"), nested = TRUE, ...)
 {
@@ -201,8 +200,7 @@ sortDicomDirectories <- function (directories, method = c("internal","divest"),
 #' 
 #' J.D. Clayden, S. Muñoz Maniega, A.J. Storkey, M.D. King, M.E. Bastin & C.A.
 #' Clark (2011). TractoR: Magnetic resonance imaging and tractography with R.
-#' Journal of Statistical Software 44(8):1-18.
-#' \url{https://www.jstatsoft.org/v44/i08/}.
+#' Journal of Statistical Software 44(8):1-18. \doi{10.18637/jss.v044.i08}.
 #' @export
 readDicomDirectory <- function (dicomDir, method = c("internal","divest"), readDiffusionParams = FALSE, untileMosaics = TRUE, ...)
 {
@@ -487,7 +485,7 @@ readDicomDirectory <- function (dicomDir, method = c("internal","divest"), readD
         {
             if (image$hasTags("effectiveReadoutTime"))
                 returnValue$echoSeparations <- image$getTags("effectiveReadoutTime")
-            else if (all(image$hasTags("echoSpacing", "epiFactor")))
+            else if (all(image$hasTags(c("echoSpacing", "epiFactor"))))
                 returnValue$echoSeparations <- image$getTags("echoSpacing") / 1e6 * (image$getTags("epiFactor") - 1)
             else
                 returnValue$echoSeparations <- rep(NA, image$nVolumes())
