@@ -28,7 +28,6 @@ getParametersForFileType <- function (fileType = NA, format = NA, singleFile = N
 identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing = TRUE, auxiliaries = c("dirs","lut","tags"), ...)
 {
     suffixes <- unique(c(.FileTypes$headerSuffixes, .FileTypes$imageSuffixes, auxiliaries))
-    fileName <- expandFileName(fileName)
     files <- ensureFileSuffix(fileName, suffixes)
     exist <- file.exists(files)
     headersExist <- intersect(unique(.FileTypes$headerSuffixes), suffixes[exist])
@@ -62,10 +61,13 @@ identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing =
     typeIndices <- which(.FileTypes$headerSuffixes == headersExist &
                          .FileTypes$imageSuffixes == imagesExist)
     
-    fileStem <- ensureFileSuffix(fileName, NULL, strip=suffixes)
+    fileStem <- expandFileName(ensureFileSuffix(fileName, NULL, strip=suffixes))
     headerFile <- ensureFileSuffix(fileStem, headersExist)
     imageFile <- ensureFileSuffix(fileStem, imagesExist)
-    auxiliaryFiles <- ensureFileSuffix(fileStem, auxiliariesExist)
+    if (is.null(auxiliaries))
+        auxiliaryFiles <- NULL
+    else
+        auxiliaryFiles <- ensureFileSuffix(fileStem, auxiliariesExist)
     
     # ANALYZE and NIFTI_PAIR file types use the same filename suffixes
     if (length(typeIndices) == 1)
@@ -73,7 +75,7 @@ identifyImageFileNames <- function (fileName, fileType = NULL, errorIfMissing =
     else if (!is.null(fileType))
         format <- (getParametersForFileType(fileType, errorIfInvalid=TRUE))$format
     else
-        format <- ifelse(hasNiftiMagicString(headerFile), "Nifti", "Analyze")
+        format <- ifelse(niftiVersion(headerFile) == 0, "Analyze", "Nifti")
     
     fileNames <- list(fileStem=fileStem, headerFile=headerFile, imageFile=imageFile, auxiliaryFiles=auxiliaryFiles, format=format, headerSuffix=headersExist, imageSuffix=imagesExist, auxiliarySuffixes=auxiliariesExist)
     return (fileNames)
@@ -160,95 +162,105 @@ copyImageFiles <- function (from, to, overwrite = FALSE, deleteOriginals = FALSE
     }
 }
 
-chooseDataTypeForImage <- function (image, format)
+chooseDataTypeForImage <- function (image, format, maxSize = NULL)
 {
     if (image$isEmpty())
-        return (NULL)
+        return ("none")
+    else if (image$isRgb())
+        return (ifelse(image$nChannels() == 4L, "rgba", "rgb"))
     else if (image$isSparse())
         data <- image$getData()$getData()
     else
         data <- image$getData()
     
     # Get the available data types for the specified format
-    datatypes <- get(paste(".",format,sep=""))$datatypes
+    datatypes <- .DatatypeCodes[[format]]
     
     # If double-mode data can be represented as integers, convert it to save space
     # Note that this slows the function down
     rType <- storage.mode(data)
-    if (rType == "double" && equivalent(as.double(data),suppressWarnings(as.integer(data))))
+    if (!is.null(maxSize) || (rType == "double" && equivalent(as.double(data),suppressWarnings(as.integer(data)))))
         rType <- "integer"
     
-    isSigned <- (rType == "double" || min(data,na.rm=TRUE) < 0)
-    
     if (rType == "double")
     {
-        singleTypeExists <- sum(datatypes$rTypes == "double" & datatypes$sizes == 4) == 1
-        doubleTypeExists <- sum(datatypes$rTypes == "double" & datatypes$sizes == 8) == 1
+        singleTypeExists <- "float" %in% names(datatypes)
+        doubleTypeExists <- "double" %in% names(datatypes)
         assert(singleTypeExists || doubleTypeExists, "Floating-point data cannot be stored using the specified file format")
         
         if (singleTypeExists && (isTRUE(getOption("tractorOutputPrecision") == "single") || !doubleTypeExists))
-            size <- 4
+            return ("float")
         else
-            size <- 8
+            return ("double")
+    }
+    else if (rType == "complex")
+    {
+        singleTypeExists <- "cfloat" %in% names(datatypes)
+        doubleTypeExists <- "cdouble" %in% names(datatypes)
+        assert(singleTypeExists || doubleTypeExists, "Complex-valued data cannot be stored using the specified file format")
         
-        isSigned <- TRUE
-        code <- datatypes$codes[datatypes$rTypes == "double" & datatypes$sizes == size]
-        name <- datatypes$names[datatypes$codes == code]
+        if (singleTypeExists && (isTRUE(getOption("tractorOutputPrecision") == "single") || !doubleTypeExists))
+            return ("cfloat")
+        else
+            return ("cdouble")
     }
     else
     {
-        compatible <- (datatypes$rTypes == "integer")
-        if (min(data,na.rm=TRUE) < 0)
-            compatible <- compatible & datatypes$isSigned
+        dataRange <- range(data, na.rm=TRUE)
+        needSigned <- (dataRange[1] < 0)
+        compatible <- names(datatypes) %~% ifelse(needSigned, "^int", "int")
         
-        maximumValues <- 2^(datatypes$sizes*8 - as.integer(datatypes$isSigned)) - 1
-        largestAbsoluteDataValue <- max(abs(max(data,na.rm=TRUE)), abs(min(data,na.rm=TRUE)))
-        compatible <- compatible & (largestAbsoluteDataValue <= maximumValues)
+        bitSizes <- as.numeric(ore_switch(names(datatypes), "\\d+$"="\\0", "0"))
+        maximumValues <- 2^(bitSizes - as.integer(names(datatypes) %~% "^int")) - 1
+        if (is.null(maxSize))
+            compatible <- compatible & (max(abs(dataRange)) <= maximumValues)
+        else
+        {
+            # If specified, the maximum datatype size must be obeyed. We then
+            # fit the data if possible; otherwise we take the highest upper
+            # bound available
+            compatible <- compatible & (bitSizes <= maxSize * 8)
+            compatible <- compatible & (max(abs(dataRange)) <= maximumValues | maximumValues == max(maximumValues[compatible]))
+        }
         
         # Prefer Analyze-compatible data types for NIfTI files
-        if (format == "Nifti" && any(compatible[datatypes$codes <= 64]))
-            compatible <- compatible & (datatypes$codes <= 64)
+        if (format == "Nifti" && any(datatypes[compatible] <= 64))
+            compatible <- compatible & (datatypes <= 64)
         
         assert(any(compatible), "No compatible data type exists for the specified image and file format")
         
         maximumValues[!compatible] <- Inf
-        code <- datatypes$codes[which.min(maximumValues)]
-        size <- datatypes$sizes[datatypes$codes == code]
-        isSigned <- datatypes$isSigned[datatypes$codes == code]
-        name <- datatypes$names[datatypes$codes == code]
+        return (names(datatypes)[which.min(maximumValues)])
     }
-    
-    return (list(code=code, type=rType, size=size, isSigned=isSigned, name=name))
 }
 
-#' Working with MRI images stored in NIfTI, Analyze and MGH formats
+#' Working with MRI images stored in various formats
 #' 
 #' Functions for reading, writing, locating, copying and removing MRI images
-#' stored in NIfTI, Analyze and MGH formats.
+#' stored in NIfTI, Analyze, MGH and MRtrix formats.
 #' 
 #' NIfTI and Analyze are related formats for storing magnetic resonance images.
 #' NIfTI is a more recent extension of Analyze, and contains more specific
 #' information about, for example, the orientation of the image. Its use is
 #' therefore recommended where possible. MGH format is used by the popular
-#' image processing package FreeSurfer. These formats use a number of different
-#' file extensions, but the details are abstracted away from the user by these
-#' functions.
+#' image processing package FreeSurfer, and MRtrix format by the software of
+#' the same name. These formats use a number of different file extensions, but
+#' the details are abstracted away from the user by these functions.
 #' 
 #' TractoR does not allow for files with the same basic name using multiple
-#' Analyze/NIfTI/MGH formats in a single directory (e.g. \code{"foo.nii"} AND
-#' \code{"foo.img"}), and these functions will produce an error if multiple
-#' compatible files exist.
+#' Analyze/NIfTI/MGH/MRtrix formats in a single directory (e.g.
+#' \code{"foo.nii"} AND \code{"foo.img"}), and these functions will produce an
+#' error if multiple compatible files exist.
 #' 
 #' Suitable values for \code{fileType} (and the \code{tractorFileType} option,
-#' which is used as a default) are \code{ANALYZE}, \code{NIFTI},
-#' \code{NIFTI_PAIR} (the two-file NIfTI format), \code{MGH},
-#' \code{ANALYZE_GZ}, \code{NIFTI_GZ}, \code{NIFTI_PAIR_GZ} and \code{MGH_GZ}.
-#' The latter four are gzipped versions of the former four. \code{NIFTI_GZ} is
-#' recommended unless there is a need for one of the others. This is the
-#' default value for the \code{tractorFileType} option, but that can be changed
-#' using a call to \code{\link{options}}, or by setting the
-#' \code{TRACTOR_FILETYPE} environment variable before loading the tractor.base
-#' package.
+#' which is used as a default for writing) are \code{"NIFTI"},
+#' \code{"NIFTI_PAIR"} (the two-file NIfTI format), \code{"MGH"}, and
+#' corresponding gzipped versions of these with \code{"_GZ"} appended. File
+#' types \code{"ANALYZE"} and \code{"MRTRIX"}, and \code{"_GZ"} variants, are
+#' additionally available for reading only. \code{"NIFTI_GZ"} is the default
+#' value for the \code{tractorFileType} option, but that can be changed using a
+#' call to \code{\link{options}}, or by setting the \code{TRACTOR_FILETYPE}
+#' environment variable before loading the \code{tractor.base} package.
 #' 
 #' Since multiple files may be involved, copying, moving or symlinking images
 #' is not trivial. \code{copyImageFiles} and \code{symlinkImageFiles} are
@@ -282,10 +294,12 @@ chooseDataTypeForImage <- function (image, format)
 #' @param overwrite Logical value: overwrite an existing image file? For
 #'   \code{writeImageFile}, an error will be raised if there is an existing
 #'   file and this is set to FALSE.
-#' @param maxSize If not \code{NULL}, the maximum number of bytes per pixel to
-#'   use when storing the data. This can lead to a substantial loss of
-#'   precision, and is usually not desirable. Only used when writing to the
-#'   NIfTI file format.
+#' @param datatype A datatype string, such as \code{"uint8"} or \code{"float"},
+#'   specifying the pixel datatype to use when storing the data. If specified,
+#'   this must be a type supported by the requested (or default) file format.
+#'   The default, \code{"fit"}, results in a datatype being chosen that is wide
+#'   enough to fit the range of the data elements. An error will arise if
+#'   there's no such type.
 #' @param writeTags Logical value: should tags be written in YAML format to an
 #'   auxiliary file?
 #' @param errorIfMissing Logical value: raise an error if no suitable files
@@ -313,14 +327,14 @@ chooseDataTypeForImage <- function (image, format)
 #'   side effects.
 #' 
 #' @author Jon Clayden
-#' @seealso The \code{RNifti} package, and \code{\linkS4class{MriImage}}.
+#' @seealso The NIfTI-1 standard (\url{http://nifti.nimh.nih.gov/nifti-1}) and
+#'   \code{\linkS4class{MriImage}}.
 #' @references Please cite the following reference when using TractoR in your
 #' work:
 #' 
 #' J.D. Clayden, S. Muñoz Maniega, A.J. Storkey, M.D. King, M.E. Bastin & C.A.
 #' Clark (2011). TractoR: Magnetic resonance imaging and tractography with R.
-#' Journal of Statistical Software 44(8):1-18.
-#' \url{https://www.jstatsoft.org/v44/i08/}.
+#' Journal of Statistical Software 44(8):1-18. \doi{10.18637/jss.v044.i08}.
 #' @rdname files
 #' @export
 readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volumes = NULL, sparse = FALSE, mask = NULL, reorder = TRUE, ...)
@@ -335,6 +349,8 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
         info <- readNifti(fileNames, metadataOnly=metadataOnly, volumes=volumes)
     else if (fileNames$format == "Mgh")
         info <- readMgh(fileNames)
+    else if (fileNames$format == "Mrtrix")
+        info <- readMrtrix(fileNames)
     else
         report(OL$Error, "Unknown image file format: #{fileNames$format}")
     
@@ -349,6 +365,7 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     # Extract some metadata
     nDims <- ndim(image)
     dims <- dim(image)
+    pixdims <- pixdim(image)
     fullDims <- c(dims, rep(1,max(0,7-nDims)))
     nVoxels <- prod(dims)
     
@@ -356,7 +373,7 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     if (!is.null(info$storage))
     {
         datatype <- info$storage$datatype
-        report(OL$Debug, "Image datatype is #{datatype$size}-byte #{ifelse(datatype$isSigned,'signed','unsigned')} #{datatype$type}")
+        report(OL$Debug, "Image datatype is #{datatype} (#{.Datatypes[[datatype]]$size}-byte #{ifelse(.Datatypes[[datatype]]$signed,'signed','unsigned')})")
     }
     report(OL$Debug, "Image orientation is #{orientation(image)}")
     
@@ -369,47 +386,27 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     willReadData <- (!metadataOnly && !RNifti:::hasData(image))
     willReorderImage <- (reorder && orientation(image) != "LAS")
     
+    assert(!willReadData || datatype %in% names(.DatatypeCodes$Nifti), "Image datatype is not supported")
+    
+    inBlockOrder <- info$storage$blockOrder %||% TRUE
+    swapEndianness <- info$storage$endian != "" && info$storage$endian != .Platform$endian
+    
     # Data has not already been read, so do it here
-    if (willReadData)
+    if (willReadData && inBlockOrder)
     {
-        # Jump to the data offset position
-        connection <- gzfile(fileNames$imageFile, "rb")
-        if (fileNames$imageFile == fileNames$headerFile)
-            readBin(connection, "raw", n=info$storage$offset)
-        
-        # Work out how many blocks (3D volumes) we're reading, and the gaps between them
+        # Work out how many blocks (3D volumes) we're reading, and the corresponding byte offsets
         blockSize <- prod(fullDims[1:3])
         if (!is.null(volumes) && nDims > 3)
-        {
             blocks <- volumes
-            jumps <- (diff(c(0, sort(volumes))) - 1) * blockSize
-        }
         else
-        {
-            blocks <- prod(fullDims[4:7])
-            jumps <- rep(0, length(blocks))
-        }
+            blocks <- seq_len(prod(fullDims[4:7]))
+        offsets <- info$storage$offset + (blocks - 1) * blockSize * .Datatypes[[datatype]]$size
         
-        willRescaleData <- (info$storage$slope != 0 && !equivalent(c(info$storage$slope,info$storage$intercept), 1:0))
         coords <- values <- NULL
         for (i in seq_along(blocks))
         {
-            # Jump over blocks being ignored, if necessary
-            if (jumps[i] > 0)
-                readBin(connection, "raw", n=jumps[i]*datatype$size)
-            
             # Read the current block
-            currentData <- array(readBin(connection, what=datatype$type, n=blockSize, size=datatype$size, signed=datatype$isSigned, endian=info$storage$endian), dim=fullDims[1:3])
-            
-            # Rescale and reorder if necessary
-            # Reordering is a purely spatial operation, so we can do it blockwise
-            if (willRescaleData)
-                currentData <- currentData * info$storage$slope + info$storage$intercept
-            if (willReorderImage)
-            {
-                currentData <- updateNifti(currentData, image)
-                orientation(currentData) <- "LAS"
-            }
+            currentData <- RNifti:::readBlob(fileNames$imageFile, blockSize, .DatatypeCodes$Nifti[datatype], offsets[i], swap=swapEndianness)
             
             if (sparse)
             {
@@ -424,11 +421,13 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
                     values <- c(values, currentData[toKeep])
                 }
             }
+            else if (length(blocks) == 1)
+                data <- currentData
             else
             {
                 # Create an array for the data if it doesn't yet exist, then insert the block
                 if (is.null(data))
-                    data <- array(as(0,datatype$type), dim=c(fullDims[1:3],length(blocks)))
+                    data <- array(as(0,.Datatypes[[datatype]]$rType), dim=c(fullDims[1:3],length(blocks)))
                 data[,,,i] <- currentData
             }
         }
@@ -439,18 +438,37 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
         
         # Update the dimensions to match the image metadata
         dim(data) <- dims
-        close(connection)
     }
+    else if (!inBlockOrder)
+    {
+        assert(length(dims) == 4, "Only 4D images out of block order are supported")
+        
+        perm <- c(2:4, 1L)
+        image$dim[2:(nDims+1)] <- dims[perm]
+        image$pixdim[2:(nDims+1)] <- pixdims[perm]
+        
+        if (willReadData)
+        {
+            data <- RNifti:::readBlob(fileNames$imageFile, nVoxels, .DatatypeCodes$Nifti[datatype], info$storage$offset, swap=swapEndianness)
+            
+            dim(data) <- dims
+            data <- aperm(data, perm)
+            
+            if (!is.null(volumes))
+                data <- data[,,,volumes]
+            if (sparse)
+                data <- as(data, "SparseArray")
+        }
+    }
+    
+    # Attach the data to the image
+    if (willReadData)
+        image <- updateNifti(data, image)
     
     # Reorder the image if requested (as opposed to the data to be associated with it)
     if (willReorderImage)
         orientation(image) <- "LAS"
     
-    # Attach the data if necessary
-    # NB: this must happen before conversion to MriImage, otherwise cal_min and cal_max won't be checked against the data
-    if (willReadData)
-        image <- updateNifti(data, image)
-    
     # Convert to an MriImage
     attr(image, "reordered") <- reorder
     image <- as(image, "MriImage")
@@ -464,6 +482,10 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     # Set the source
     image$setSource(fileNames$fileStem)
     
+    # If the reader produced any tags, set them first (auxiliary files will overwrite duplicates)
+    if (!is.null(info$tags))
+        do.call(image$setTags, as.list(info$tags))
+    
     # Read the auxiliary tags file, if one exists
     tagsFileName <- ensureFileSuffix(fileNames$fileStem, "tags")
     if (file.exists(tagsFileName))
@@ -485,14 +507,17 @@ readImageFile <- function (fileName, fileType = NULL, metadataOnly = FALSE, volu
     return (image)
 }
 
-writeImageData <- function (image, connection, type, size, endian = .Platform$endian)
+writeImageData <- function (image, connection, datatype, endian = .Platform$endian)
 {
+    # RNifti handles RGB(A) for NIfTI/ANALYZE; no other write format supports it
+    assert(!(datatype %in% c("rgb","rgba")), "Cannot write RGB data at the moment")
+    
     image <- as(image, "MriImage")
     data <- image$getData()
     dims <- image$getDimensions()
     
     # writeBin() can only write 2^31 - 1 bytes, so work blockwise if necessary
-    if (image$isSparse() || prod(dims) * size >= 2^31)
+    if (image$isSparse() || prod(dims) * .Datatypes[[datatype]]$size >= 2^31)
     {
         nDims <- image$getDimensionality()
         for (i in seq_len(dims[nDims]))
@@ -501,16 +526,16 @@ writeImageData <- function (image, connection, type, size, endian = .Platform$en
             indices[[nDims]] <- i
             currentData <- as.array(do.call("[", c(list(data),indices)))
             
-            storage.mode(currentData) <- type
+            storage.mode(currentData) <- .Datatypes[[datatype]]$rType
             attributes(currentData) <- NULL
-            writeBin(currentData, connection, size=size, endian=endian)
+            writeBin(currentData, connection, size=.Datatypes[[datatype]]$size, endian=endian)
         }
     }
     else
     {
-        storage.mode(data) <- type
+        storage.mode(data) <- .Datatypes[[datatype]]$rType
         attributes(data) <- NULL
-        writeBin(data, connection, size=size, endian=endian)
+        writeBin(data, connection, size=.Datatypes[[datatype]]$size, endian=endian)
     }
 }
 
@@ -525,7 +550,7 @@ writeGradientDirections <- function (directions, bValues, fileName)
 
 #' @rdname files
 #' @export
-writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = TRUE, maxSize = NULL, writeTags = FALSE)
+writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = TRUE, datatype = "fit", writeTags = FALSE)
 {
     image <- as(image, "MriImage")
     
@@ -561,11 +586,18 @@ writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = T
     if (params$format == "Analyze")
         report(OL$Error, "Writing to ANALYZE format is no longer supported")
     else if (params$format == "Nifti")
-        writeNifti(image, fileNames, maxSize=maxSize)
+        writeNifti(image, fileNames, datatype=datatype)
     else if (params$format == "Mgh")
-        writeMgh(image, fileNames, gzipped=params$gzipped)
+        writeMgh(image, fileNames, datatype=datatype, gzipped=params$gzipped)
     
-    if (writeTags && image$nTags() > 0)
+    if (image$isInternal())
+    {
+        image$setSource(expandFileName(fileStem))
+        if (Sys.getenv("TRACTOR_COMMANDLINE") != "")
+            image$setTags(commandHistory=Sys.getenv("TRACTOR_COMMANDLINE"), merge=TRUE)
+    }
+    
+    if ((writeTags || file.exists(ensureFileSuffix(fileStem,"tags"))) && image$nTags() > 0)
     {
         tags <- image$getTags()
         if (all(c("bVectors","bValues") %in% names(tags)))
@@ -575,8 +607,5 @@ writeImageFile <- function (image, fileName = NULL, fileType = NA, overwrite = T
             writeLines(yaml::as.yaml(tags,handlers=list(Date=format.Date)), ensureFileSuffix(fileStem,"tags"))
     }
     
-    if (image$isInternal())
-        image$setSource(expandFileName(fileStem))
-    
     invisible (fileNames)
 }
